@@ -1,12 +1,13 @@
 package String::Util;
+
 use strict;
+use warnings;
 use Carp;
-use overload;
-use 5.010;
+use v5.14;
 
 # version
-our $VERSION = '1.30';
-
+our $VERSION  = '1.33';
+our $FGC_MODE = 'UTF-8';
 
 #------------------------------------------------------------------------------
 # opening POD
@@ -14,21 +15,26 @@ our $VERSION = '1.30';
 
 =head1 NAME
 
-String::Util -- String processing utility functions
+B<String::Util> -- String processing utility functions
 
 =head1 DESCRIPTION
 
-String::Util provides a collection of small, handy functions for processing
+B<String::Util> provides a collection of small, handy functions for processing
 strings in various ways.
 
 =head1 INSTALLATION
 
-String::Util can be installed with the usual routine:
+  cpanm String::Util
 
-  perl Makefile.PL
-  make
-  make test
-  make install
+=head1 USAGE
+
+No functions are exported by default, they must be specified:
+
+  use String::Util qw(trim eqq contains)
+
+alternately you can use C<:all> to export B<all> of the functions
+
+  use String::Util qw(:all)
 
 =head1 FUNCTIONS
 
@@ -49,21 +55,18 @@ use vars qw[@EXPORT_OK %EXPORT_TAGS];
 # the following functions accept a value and return a modified version of
 # that value
 push @EXPORT_OK, qw[
-	collapse     crunch     htmlesc    trim      ltrim
-	rtrim        define     repeat     unquote   no_space
-	nospace      fullchomp  randcrypt  jsquote   cellfill
-	crunchlines
+	collapse     htmlesc    trim      ltrim
+	rtrim        repeat     unquote   no_space
+	nospace      jsquote    crunchlines
+	file_get_contents
 ];
 
 # the following functions return true or false based on their input
 push @EXPORT_OK, qw[
-	hascontent  nocontent eqq equndef neqq neundef
-	startswith  endswith  contains    sanitize
+	hascontent  nocontent eqq      neqq
+	startswith  endswith  contains sanitize
 ];
 
-# the following function returns a random string of some type
-push @EXPORT_OK, qw[ randword ];
-
 # the following function returns the unicode values of a string
 push @EXPORT_OK, qw[ ords deords ];
 
@@ -83,9 +86,6 @@ C<collapse()> collapses all whitespace in the string down to single spaces.
 Also removes all leading and trailing whitespace.  Undefined input results in
 undefined output.
 
-B<Note:> C<crunch()> is an alias to this function. It is considered deprecated.
-It may be removed in future versions.
-
   $var = collapse("  Hello     world!    "); # "Hello world!"
 
 =cut
@@ -102,9 +102,6 @@ sub collapse {
 	return $val;
 }
 
-sub crunch {
-	return collapse(@_);
-}
 #
 # collapse
 #------------------------------------------------------------------------------
@@ -116,7 +113,7 @@ sub crunch {
 
 =head2 hascontent($scalar), nocontent($scalar)
 
-hascontent() returns true if the given argument is defined and contains
+C<hascontent()> returns true if the given argument is defined and contains
 something besides whitespace.
 
 An undefined value returns false.  An empty string returns false.  A value
@@ -136,16 +133,27 @@ C<nocontent()> returns the negation of C<hascontent()>.
 =cut
 
 sub hascontent {
-	my ($val) = @_;
+	my $val = shift();
 
-	defined($val) or return 0;
-	$val =~ m|\S|s or return 0;
+	if (!defined($val)) {
+		return 0;
+	}
 
-	return 1;
+	# If there are ANY non-space characters in it
+	if ($val =~ m|\S|s) {
+		return 1;
+	}
+
+	return 0;
 }
 
 sub nocontent {
-	return ! hascontent(@_);
+	my $str = shift();
+
+	# nocontent() is just the inverse to hascontent()
+	my $ret = !(hascontent($str));
+
+	return $ret;
 }
 
 #
@@ -160,25 +168,24 @@ sub nocontent {
 =head2 trim($string), ltrim($string), rtrim($string)
 
 Returns the string with all leading and trailing whitespace removed.
-Trim on undef returns "".
 
   $var = trim(" my string  "); # "my string"
 
-ltrim() trims B<leading> whitespace only.
+C<ltrim()> trims B<leading> whitespace only.
 
-rtrim() trims B<trailing> whitespace only.
+C<rtrim()> trims B<trailing> whitespace only.
 
 =cut
 
 sub trim {
 	my $s = shift();
 
-	if (!defined($s) || length($s) == 0) {
-		return "";
+	if (!defined($s)) {
+		return undef;
 	}
 
-	$s =~ s/^\s*//;
-	$s =~ s/\s*$//;
+	$s =~ s/^\s*//u;
+	$s =~ s/\s*$//u;
 
 	return $s;
 }
@@ -194,11 +201,11 @@ sub trim {
 sub ltrim {
 	my $s = shift();
 
-	if (!defined($s) || length($s) == 0) {
-		return "";
+	if (!defined($s)) {
+		return undef;
 	}
 
-	$s =~ s/^\s*//;
+	$s =~ s/^\s*//u;
 
 	return $s;
 }
@@ -206,11 +213,11 @@ sub ltrim {
 sub rtrim {
 	my $s = shift();
 
-	if (!defined($s) || length($s) == 0) {
-		return "";
+	if (!defined($s)) {
+		return undef;
 	}
 
-	$s =~ s/\s*$//;
+	$s =~ s/\s*$//u;
 
 	return $s;
 }
@@ -225,26 +232,29 @@ sub rtrim {
 # no_space
 #
 
-=head2 no_space($string)
+=head2 nospace($string)
 
 Removes B<all> whitespace characters from the given string. This includes spaces
 between words.
 
-  $var = no_space("  Hello World!   "); # "HelloWorld!"
+  $var = nospace("  Hello World!   "); # "HelloWorld!"
 
 =cut
 
 sub no_space {
-	my ($val) = @_;
-
-	if (defined $val)
-		{ $val =~ s|\s+||gs }
-
-	return $val;
+	return nospace(@_);
 }
 
 # alias nospace to no_space
-sub nospace { return no_space(@_) }
+sub nospace {
+	my $val = shift();
+
+	if (defined $val) {
+		$val =~ s|\s+||gs;
+	}
+
+	return $val;
+}
 
 #
 # no_space
@@ -256,7 +266,7 @@ sub nospace { return no_space(@_) }
 # htmlesc
 #
 
-=head2 htmlesc(string)
+=head2 htmlesc($string)
 
 Formats a string for literal output in HTML.  An undefined value is returned as
 an empty string.
@@ -275,8 +285,7 @@ sub htmlesc {
 		$val =~ s|\"|&quot;|g;
 		$val =~ s|\<|&lt;|g;
 		$val =~ s|\>|&gt;|g;
-	}
-	else {
+	} else {
 		$val = '';
 	}
 
@@ -287,30 +296,6 @@ sub htmlesc {
 #------------------------------------------------------------------------------
 
 
-#------------------------------------------------------------------------------
-# cellfill
-#
-
-sub cellfill{
-	my ($val) = @_;
-
-	carp("cellfill() is deprecated and will be removed in future versions");
-
-	if (hascontent($val)) {
-		$val = htmlesc($val);
-	}
-
-	else {
-		$val = '&nbsp;';
-	}
-
-	return $val;
-}
-#
-# cellfill
-#------------------------------------------------------------------------------
-
-
 #------------------------------------------------------------------------------
 # jsquote
 #
@@ -325,6 +310,10 @@ surrounds the string in single quotes.  Returns the modified string.
 sub jsquote {
 	my ($str) = @_;
 
+	if (!defined($str)) {
+		return undef;
+	}
+
 	# Escape single quotes.
 	$str =~ s|'|\\'|gs;
 
@@ -397,30 +386,6 @@ sub unquote {
 #------------------------------------------------------------------------------
 
 
-#------------------------------------------------------------------------------
-# define
-#
-
-sub define {
-	carp("define() is deprecated and may be removed in future version");
-
-	my ($val) = @_;
-
-	# if overloaded object, get return value and
-	# concatenate with string (which defines it).
-	if (ref($val) && overload::Overloaded($val)) {
-		local $^W = 0;
-		$val = $val . '';
-	}
-
-	defined($val) or $val = '';
-	return $val;
-}
-#
-# define
-#------------------------------------------------------------------------------
-
-
 #------------------------------------------------------------------------------
 # repeat
 #
@@ -432,8 +397,8 @@ command outputs "Fred" three times:
 
   print repeat('Fred', 3), "\n";
 
-Note that repeat() was created a long time based on a misunderstanding of how
-the perl operator 'x' works.  The following command using 'x' would perform
+Note that C<repeat()> was created a long time based on a misunderstanding of how
+the perl operator 'x' works.  The following command using C<x> would perform
 exactly the same as the above command.
 
   print 'Fred' x 3, "\n";
@@ -451,195 +416,6 @@ sub repeat {
 #------------------------------------------------------------------------------
 
 
-#------------------------------------------------------------------------------
-# randword
-#
-
-=head2 randword($length, %options)
-
-Returns a random string of characters. String will not contain any vowels (to
-avoid distracting dirty words). First argument is the length of the return
-string. So this code:
-
-  foreach my $idx (1..3) {
-      print randword(4), "\n";
-  }
-
-would output something like this:
-
-  kBGV
-  NCWB
-  3tHJ
-
-If the string 'dictionary' is sent instead of an integer, then a word is
-randomly selected from a dictionary file.  By default, the dictionary file
-is assumed to be at /usr/share/dict/words and the shuf command is used to
-pull out a word.  The hash %String::Util::PATHS sets the paths to the
-dictionary file and the shuf executable.  Modify that hash to change the paths.
-So this code:
-
-  foreach my $idx (1..3) {
-      print randword('dictionary'), "\n";
-  }
-
-would output something like this:
-
-  mustache
-  fronds
-  browning
-
-B<option:> alpha
-
-If the alpha option is true, only alphabetic characters are returned, no
-numerals. For example, this code:
-
-  foreach my $idx (1..3) {
-      print randword(4, alpha=>1), "\n";
-  }
-
-would output something like this:
-
-  qrML
-  wmWf
-  QGvF
-
-B<option:> numerals
-
-If the numerals option is true, only numerals are returned, no alphabetic
-characters. So this code:
-
-  foreach my $idx (1..3) {
-      print randword(4, numerals=>1), "\n";
-  }
-
-would output something like this:
-
-  3981
-  4734
-  2657
-
-B<option:> strip_vowels
-
-This option is true by default.  If true, vowels are not included in the
-returned random string. So this code:
-
-  foreach my $idx (1..3) {
-      print randword(4, strip_vowels=>1), "\n";
-  }
-
-would output something like this:
-
-  Sk3v
-  pV5z
-  XhSX
-
-=cut
-
-# path information for WC
-our %PATHS = (
-	wc => '/usr/bin/wc',
-	shuf => '/usr/bin/shuf',
-	words => '/usr/share/dict/words',
-	head => '/usr/bin/head',
-	tail => '/usr/bin/tail',
-);
-
-sub randword {
-	my ($count, %opts) = @_;
-	my ($rv, $char, @chars, $paths);
-	$rv = '';
-
-	# check syntax
-	defined($count) or croak 'syntax: randword($count)';
-
-	# dictionary word
-	if ($count =~ m|^dict|si) {
-		# loop until acceptable word is found
-		DICTIONARY:
-		while (1) {
-			my ($cmd, $line_count, $line_num, $word);
-
-			# use Encode module
-			require Encode;
-
-			# get line count
-			$cmd = qq|$PATHS{'wc'} -l "$PATHS{'words'}"|;
-			($line_count) = `$cmd`;
-			$line_count =~ s|\s.*||s;
-
-			# get random line
-			$line_num = rand($line_count);
-
-			# untaint line number
-			unless ($line_num =~ m|^([0-9]+)$|s) { die "invalid line number: $line_num" }
-			$line_num = $1;
-
-			# get random word
-			$cmd = qq[$PATHS{'head'} -$line_num "$PATHS{'words'}" | $PATHS{'tail'} -1];
-			($word) = `$cmd`;
-			$word =~ s|\s.*||si;
-			$word =~ s|'.*||si;
-			$word = lc($word);
-
-			# only allow words that are all letters
-			if ($opts{'letters_only'}) {
-				unless ($word =~ m|^[a-z]+$|s)
-					{ next DICTIONARY }
-			}
-
-			# check for max length
-			if ($opts{'maxlength'}) {
-				if ( length($word) > $opts{'maxlength'} )
-					{ next DICTIONARY }
-			}
-
-			# encode unless specifically opted not to do so
-			unless ( defined($opts{'encode'}) && (! $opts{'encode'}) )
-				{ $word = Encode::encode_utf8($word) }
-
-			# return
-			return $word;
-		}
-	}
-
-	# alpha only
-	if ($opts{'alpha'})
-		{ @chars = ('a' .. 'z', 'A' .. 'Z') }
-
-	# else alpha and numeral
-	else
-		{ @chars = ('a' .. 'z', 'A' .. 'Z', '0' .. '9') }
-
-	# defaults
-	defined($opts{'strip_vowels'}) or $opts{'strip_vowels'} = 1;
-
-	while (length($rv) < $count) {
-		$char = rand();
-
-		# numerals only
-		if ($opts{'numerals'}) {
-			$char =~ s|^0.||;
-			$char =~ s|\D||g;
-		}
-
-		# character random word
-		else {
-			$char = int( $char * ($#chars + 1) );
-			$char = $chars[$char];
-			next if($opts{'strip_vowels'} && $char =~ m/[aeiouy]/i);
-		}
-
-		$rv .= $char;
-	}
-
-	return substr($rv, 0, $count);
-}
-#
-# randword
-#------------------------------------------------------------------------------
-
-
-
 #------------------------------------------------------------------------------
 # eqq
 # formerly equndef
@@ -648,31 +424,27 @@ sub randword {
 =head2 eqq($scalar1, $scalar2)
 
 Returns true if the two given values are equal.  Also returns true if both
-are undef.  If only one is undef, or if they are both defined but different,
+are C<undef>.  If only one is C<undef>, or if they are both defined but different,
 returns false. Here are some examples and what they return.
 
-  $var = eqq('x', 'x'), "\n";      # True
-  $var = eqq('x', undef), "\n";    # False
-  $var = eqq(undef, undef), "\n";  # True
-
-B<Note:> equndef() is an alias to this function. It is considered deprecated.
-It may be removed in future versions.
+  $var = eqq('x', 'x');     # True
+  $var = eqq('x', undef);   # False
+  $var = eqq(undef, undef); # True
 
 =cut
 
-# alias equndef to eqq
-sub equndef { return eqq(@_) }
-
 sub eqq {
 	my ($str1, $str2) = @_;
 
 	# if both defined
-	if ( defined($str1) && defined($str2) )
-		{ return $str1 eq $str2 }
+	if ( defined($str1) && defined($str2) ) {
+		return $str1 eq $str2
+	}
 
 	# if neither are defined
-	if ( (! defined($str1)) && (! defined($str2)) )
-		{ return 1 }
+	if ( (!defined($str1)) && (!defined($str2)) ) {
+		return 1
+	}
 
 	# only one is defined, so return false
 	return 0;
@@ -689,22 +461,15 @@ sub eqq {
 
 =head2 neqq($scalar1, $scalar2)
 
-The opposite of neqq, returns true if the two values are *not* the same.
+The opposite of C<neqq>, returns true if the two values are *not* the same.
 Here are some examples and what they return.
 
-  $var = neqq('x', 'x'), "\n";      # False
-  $var = neqq('x', undef), "\n";    # True
-  $var = neqq(undef, undef), "\n";  # False
-
-B<Note:> neundef() is an alias to this function. It is considered deprecated.
-It may be removed in future versions.
+  $var = neqq('x', 'x');     # False
+  $var = neqq('x', undef);   # True
+  $var = neqq(undef, undef); # False
 
 =cut
 
-sub neundef {
-	return eqq(@_) ? 0 : 1;
-}
-
 sub neqq {
 	return eqq(@_) ? 0 : 1;
 }
@@ -713,45 +478,6 @@ sub neqq {
 #------------------------------------------------------------------------------
 
 
-#------------------------------------------------------------------------------
-# fullchomp
-#
-
-sub fullchomp {
-	carp("fullchomp() is deprecated and may be removed in future versions");
-
-	my ($line) = @_;
-	defined($line) and $line =~ s|[\r\n]+$||s;
-	defined(wantarray) and return $line;
-	$_[0] = $line;
-}
-#
-# fullchomp
-#------------------------------------------------------------------------------
-
-
-#------------------------------------------------------------------------------
-# randcrypt
-#
-
-=head2 randcrypt($string)
-
-Crypts the given string, seeding the encryption with a random two character
-seed.
-
-=cut
-
-sub randcrypt {
-	my ($pw) = @_;
-	my ($rv);
-	$rv = crypt($pw, randword(2));
-	return $rv;
-}
-#
-# randcrypt
-#------------------------------------------------------------------------------
-
-
 #------------------------------------------------------------------------------
 # ords
 #
@@ -845,7 +571,7 @@ sub ords {
 
 =head2 deords($string)
 
-Takes the output from ords() and returns the string that original created that
+Takes the output from C<ords()> and returns the string that original created that
 output.
 
   $var = deords('{72}{101}{110}{100}{114}{105}{120}'); # 'Hendrix'
@@ -881,11 +607,17 @@ Checks if the string contains substring
   $var = contains("Hello world", "llo wor"); # true
   $var = contains("Hello world", "QQQ");     # false
 
+  # Also works with grep
+  @arr = grep { contains("cat") } @input;
+
 =cut
 
 sub contains {
-	my $str    = shift() || "";
-	my $substr = shift();
+	my ($str, $substr) = @_;
+
+	if (!defined($str)) {
+		return undef;
+	}
 
 	if (!$substr) {
 		$substr = $str;
@@ -901,15 +633,21 @@ sub contains {
 
 Checks if the string starts with the characters in substring
 
-  $var = startwidth("Hello world", "Hello"); # true
-  $var = startwidth("Hello world", "H");     # true
-  $var = startwidth("Hello world", "Q");     # false
+  $var = startwith("Hello world", "Hello"); # true
+  $var = startwith("Hello world", "H");     # true
+  $var = startwith("Hello world", "Q");     # false
+
+  # Also works with grep
+  @arr = grep { startswith("X") } @input;
 
 =cut
 
 sub startswith {
-	my $str    = shift() || "";
-	my $substr = shift();
+	my ($str, $substr) = @_;
+
+	if (!defined($str)) {
+		return undef;
+	}
 
 	if (!$substr) {
 		$substr = $str;
@@ -921,43 +659,25 @@ sub startswith {
 	return $ret;
 }
 
-=head2 sanitize($string)
-
-Sanitize all non alpha-numeric characters in a string to underscores.
-This is useful to take a URL, or filename, or text description and know
-you can use it safely in a URL or a filename.
-
-B<Note:> This will remove any trailing or leading '_' on the string
-
-  $var = sanitize("http://www.google.com/") # http_www_google_com
-  $var = sanitize("foo_bar()";              # foo_bar
-  $var = sanitize("/path/to/file.txt");     # path_to_file_txt
-
-=cut
-
-sub sanitize {
-	my $str = shift();
-
-	$str =~ s/[\W_]+/_/g;
-	$str =~ s/\A_+//g;
-	$str =~ s/_+\z//g;
-
-	return $str;
-}
-
 =head2 endswith($string, $substring)
 
 Checks if the string ends with the characters in substring
 
-  $var = endswidth("Hello world", "world");   # true
-  $var = endswidth("Hello world", "d");       # true
-  $var = endswidth("Hello world", "QQQ");     # false
+  $var = endswith("Hello world", "world");   # true
+  $var = endswith("Hello world", "d");       # true
+  $var = endswith("Hello world", "QQQ");     # false
+
+  # Also works with grep
+  @arr = grep { endswith("z") } @input;
 
 =cut
 
 sub endswith {
-	my $str    = shift() || "";
-	my $substr = shift();
+	my ($str, $substr) = @_;
+
+	if (!defined($str)) {
+		return undef;
+	}
 
 	if (!$substr) {
 		$substr = $str;
@@ -989,6 +709,10 @@ single newline.
 sub crunchlines {
 	my ($str) = @_;
 
+	if (!defined($str)) {
+		return undef;
+	}
+
 	while($str =~ s|\n[ \t]*\n|\n|gs)
 		{}
 
@@ -1001,6 +725,84 @@ sub crunchlines {
 # crunchlines
 #------------------------------------------------------------------------------
 
+=head2 sanitize($string, $separator = "_")
+
+Sanitize all non alpha-numeric characters in a string to underscores.
+This is useful to take a URL, or filename, or text description and know
+you can use it safely in a URL or a filename.
+
+B<Note:> This will remove any trailing or leading '_' on the string
+
+  $var = sanitize("http://www.google.com/") # http_www_google_com
+  $var = sanitize("foo_bar()";              # foo_bar
+  $var = sanitize("/path/to/file.txt");     # path_to_file_txt
+  $var = sanitize("Big yellow bird!", "."); # Big.yellow.bird
+
+=cut
+
+sub sanitize {
+    my $str = shift();
+    my $sep = shift() // "_";
+
+    if (!defined($str)) {
+        return undef;
+    }
+
+    # Convert multiple non-word sequences to the separator
+    $str =~ s/[\W_]+/$sep/g;
+
+    # The separator is a literal character so we quotemeta it
+    $sep = quotemeta($sep);
+    # Remove any separators at the beginning and end
+    $str =~ s/\A$sep+//;
+    $str =~ s/$sep+\z//;
+
+    return $str;
+}
+
+###########################################################################
+
+=head2 file_get_contents($string, $boolean)
+
+Read an entire file from disk into a string. Returns undef if the file
+cannot be read for any reason. Can also return the file as an array of
+lines.
+
+  $str   = file_get_contents("/tmp/file.txt");    # Return a string
+  @lines = file_get_contents("/tmp/file.txt", 1); # Return an array
+
+B<Note:> If you opt to return an array, carriage returns and line feeds are
+removed from the end of each line.
+
+B<Note:> File is read in B<UTF-8> mode, unless C<$FGC_MODE> is set to an
+appropriate encoding.
+
+=cut
+
+sub file_get_contents {
+	my ($file, $ret_array) = @_;
+
+	open (my $fh, "<", $file) or return undef;
+	binmode($fh, ":encoding($FGC_MODE)");
+
+	if ($ret_array) {
+		my @ret;
+
+		while (my $line = readline($fh)) {
+			$line =~ s/[\r\n]*$//; # Remove CR/LF
+			push(@ret, $line);
+		}
+
+		return @ret;
+	} else {
+		my $ret = '';
+		while (my $line = readline($fh)) {
+			$ret .= $line;
+		}
+
+		return $ret;
+	}
+}
 
 # return true
 1;
